using TMPro;
using UnityEngine;
using System.Linq;
using System;

namespace Mahjong
{
    [Serializable]
    public struct EvaluateData
    {
        public int Grade;
        public bool Is_Yukman;
        public static EvaluateData By(int grade, bool is_yukman)
        {
            var result = new EvaluateData();
            result.Grade = grade;
            result.Is_Yukman = is_yukman;
            return result;
        }
    }
    public class Evaluator : MonoBehaviour
    {
        private TMP_Text resultText => GameManager.Instance.Prompt;
        private HandData handData => GameManager.Instance.HandData;
        private GameData gameData => GameManager.Instance.GameData;
        private bool menzen;

        public EvaluateData Evaluate(bool updateText = false)
        {
            string result = "";
            string resultYukman = "";
            int dora = 0;
            int grade = 0;
            int gradeYukman = 0;
            menzen = Is_Menzen(handData);

            if (Is_MenzenTsumo())
            {
                result += ", ";
                grade += 1;
            }

            if (Count_Dora() > 0)
            {
                result += $" * {Count_Dora()}, ";
                dora += Count_Dora();
            }

            if (gameData.Rich)
            {
                if (gameData.OneBal)
                {
                    result += "ġ Ϲ, ";
                    grade += 2;
                }
                else
                {
                    result += "ġ, ";
                    grade += 1;
                }

                if (Count_Backdora() > 0)
                {
                    result += $"޵ * {Count_Backdora()}, ";
                    dora += Count_Backdora();
                }
            }

            if (Is_Pinhu())
            {
                result += ", ";
                grade += 1;
            }
            
            if (Is_Ipeko())
            {
                if(Is_Ryanpeko())
                {
                    result += ", ";
                    grade += 3;
                }
                else
                {
                    result += ", ";
                    grade += 1;
                }
            }

            if (Is_Tanyao())
            {
                result += "߿, ";
                grade += 1;
            }

            if (Is_Youngsang())
            {
                result += "ȭ, ";
                grade += 1;
            }

            // â, ο, ξ -  ߰ؾϳ?

            switch (Count_Color())
            {
                case 1:
                    result += ", ";
                    grade += 1;
                    break;
                case 2:
                    result += " * 2, ";
                    grade += 2;
                    break;
                case 3:
                    resultYukman += ", ";
                    gradeYukman += 1;
                    break;
            }

            if (Same_GameDirection())
            {
                result += "ǳ, ";
                grade += 1;
            }

            if (Same_PlayerDirection())
            {
                result += "ǳ, ";
                grade += 1;
            }

            //2ǿ

            // ġ

            if (Is_ShunziWithThreeTypes())
            {
                if (menzen)
                {
                    result += "(), ";
                    grade += 2;
                }
                else
                {
                    result += "(ķ), ";
                    grade += 1;
                }
            }

            if(Is_KeziWithThreeTypes())
            {
                result += ", ";
                grade += 2;
            }

            if(Is_Chanta())
            {
                if (Is_Honnodu())
                {
                    result += "ȥ, ";
                    grade += 2;
                }
                else
                {
                    if (menzen)
                    {
                        result += "Ÿ(), ";
                        grade += 2;
                    }
                    else
                    {
                        result += "Ÿ(ķ), ";
                        grade += 1;
                    }
                        
                }
                
            }

            if (Is_IncludingOneToNine())
            {
                if (menzen)
                {
                    result += "ϱ(), ";
                    grade += 2;
                }
                else
                {
                    result += "ϱ(ķ), ";
                    grade += 1;
                }
            }

            if (Count_Kezi() == 4 && !menzen)
            {
                result += "̶, ";
                grade += 2;
            }

            

            if (Count_AnKezi() >= 3)
            {
                if(Count_AnKezi() == 4 && menzen)
                {
                    if(handData.LastHand.LastHandFrom == 4)
                    {
                        resultYukman += "Ŀ ܱ, ";
                        gradeYukman += 2;
                    }
                    else
                    {
                        resultYukman += "Ŀ, ";
                        gradeYukman += 1;
                    }
                }
                else
                {
                    result += "Ŀ, ";
                    grade += 2;
                }
            }

            if (Is_SmallTripleColor())
            {
                result += "һ, ";
                grade += 2;
            }

            if (Count_Gangzi() >= 3)
            {
                if(Count_Gangzi() >= 4)
                {
                    resultYukman += ", ";
                    gradeYukman += 1;
                }
                else
                {
                    result += ". ";
                    grade += 2;
                }
            }

            if (Is_IncludingOneOrNine())
            {
                if (menzen)
                {
                    result += "Ÿ(), ";
                    grade += 3;
                }
                else
                {
                    result += "Ÿ(ķ), ";
                    grade += 2;
                }
            }

            if (Is_ComplexOneType())
            {
                if(Count_Char() == 5)
                {
                    resultYukman += "ϻ, ";
                    gradeYukman += 1;
                }
                else if (Is_PureOneType())
                {
                    result += "ûϻ, ";
                    grade += 6;
                }
                else
                {
                    if (menzen)
                    {
                        result += "ȥϻ(), ";
                        grade += 3;
                    }
                    else
                    {
                        result += "ȥϻ(ķ), ";
                        grade += 2;
                    }
                }
            }

            if(Is_OnlyGreen())
            {
                resultYukman += "ϻ, ";
                gradeYukman += 1;
            }

            if(Is_Chungnodu())
            {
                resultYukman += "û, ";
                gradeYukman += 1;
            }

            if(Is_Smallsahi())
            {
                resultYukman += "һ, ";
                gradeYukman += 1;
            }

            if(Is_Bigsahi())
            {
                resultYukman += ", ";
                gradeYukman += 2;
            }

            if(Is_ConnectedNine())
            {
                if(Is_PureConnectedNine())
                {
                    resultYukman += " ú, ";
                    gradeYukman += 2;
                }
                else
                {
                    resultYukman += "ú, ";
                    gradeYukman += 1;
                }
            }

            
            if (gradeYukman > 0)
            {
                if(updateText)
                    this.resultText.text = resultYukman + $" * {gradeYukman}";
                return EvaluateData.By(gradeYukman, true);
            }
            else if (grade == 0)
            {
                if(updateText)
                    this.resultText.text = " ";
                return EvaluateData.By(0, false);
            }
            else
            {
                grade += dora;
                if(grade >= 13)
                {
                    gradeYukman += 1;
                    if(updateText)
                        this.resultText.text= result + "ī ";
                    return EvaluateData.By(gradeYukman, true);
                }
                else
                {
                    if(updateText)
                        this.resultText.text = result + $"{grade}";
                    return EvaluateData.By(grade, false);
                }
            }
                

        }

        public static bool Is_Menzen(HandData handData)
        {
            foreach(var handStruct in handData.StructList)
            {
                switch (handStruct.StructType)
                {
                    case HandStructType.Pun:
                        return false;
                    case HandStructType.Chi:
                        return false;
                    case HandStructType.Gangzi_Out:
                        return false;
                }
            }
            
            return true;
        }

        public static int Count_Gangzi_Static(HandData handData)
        {
            int count_gangzi = 0;
            foreach (var handStruct in handData.StructList)
            {
                if (Is_Gangzi_Static(handStruct))
                    count_gangzi += 1;
            }
            return count_gangzi;
        }

        public static bool Is_Gangzi_Static(HandStruct handStruct)
        {
            return handStruct.StructType == HandStructType.Gangzi_In || handStruct.StructType == HandStructType.Gangzi_Out;
        }

        private bool Is_Shunzi(HandStruct handStruct)
        {
            return handStruct.StructType == HandStructType.Shunzi || handStruct.StructType == HandStructType.Chi;
        }

        private bool Is_Kezi(HandStruct handStruct)
        {
            return handStruct.StructType == HandStructType.Kezi || handStruct.StructType == HandStructType.Pun || handStruct.StructType == HandStructType.Gangzi_In || handStruct.StructType == HandStructType.Gangzi_Out;
        }

        private int Count_AnKezi()
        {
            int count = 0;
            for(int i=0; i<5;i++)
            {
                var handStruct = handData.StructList[i];
                if (i == handData.LastHand.LastHandFrom && handData.LastHand.LastHandType == LastHandType.Ron)
                    continue;
                else if (handStruct.StructType == HandStructType.Kezi || handStruct.StructType == HandStructType.Gangzi_In)
                    count += 1;
            }
            return count;
        }

        private bool Is_Gangzi(HandStruct handStruct)
        {
            return handStruct.StructType == HandStructType.Gangzi_In || handStruct.StructType == HandStructType.Gangzi_Out;
        }    

        private bool Is_NumberHand(HandStruct handStruct, int num)
        {
            return (int)handStruct.HandType % 10 == num && (int)handStruct.HandType > 10;
        }

        private bool Is_CharHand(HandStruct handStruct)
        {
            return (int)handStruct.HandType < 10;
        }

        private bool Is_ColorHand(HandStruct handStruct)
        {
            return handStruct.HandType == HandType.White || handStruct.HandType == HandType.Green || handStruct.HandType == HandType.Red;
        }

        private bool Is_RequireHand(HandStruct handStruct)
        {
            int[] requireHands = { 1, 2, 3, 4, 5, 6, 7, 11, 19, 21, 29, 31, 39 };
            return requireHands.Contains((int)handStruct.HandType);
        }

        private bool Is_MenzenTsumo()
        {
            if (menzen && handData.LastHand.LastHandType != LastHandType.Ron)
                return true;
            else
                return false;
        }

        private int Count_HandAmount(HandStruct handStruct, HandType handType)
        {
            if (Is_Gangzi(handStruct) && handStruct.HandType == handType)
                return 4;
            else if (Is_Kezi(handStruct) && handStruct.HandType == handType)
                return 3;
            else if (handStruct.StructType == HandStructType.Head && handStruct.HandType == handType)
                return 2;
            else if (Is_Shunzi(handStruct) && ((int)handType - (int)handStruct.HandType) <= 2 && ((int)handType - (int)handStruct.HandType) >= 0)
                return 1;
            else
                return 0;

        }

        private int Count_Dora()
        {
            int count_dora = 0;
            foreach(var handStruct in handData.StructList)
            {
                foreach(var dora in gameData.DoraList)
                {
                    count_dora += Count_HandAmount(handStruct, dora);
                }
            }
            return count_dora;
        }

        private int Count_Backdora()
        {
            int count_dora = 0;
            foreach (var handStruct in handData.StructList)
            {
                foreach (var dora in gameData.BackdoraList)
                {
                    count_dora += Count_HandAmount(handStruct, dora);
                }
            }
            return count_dora;
        }


        private bool Is_Pinhu()
        {
            if (!menzen)
                return false;
            if (Count_Kezi() > 0)
                return false;

            foreach(var handStruct in handData.StructList)
            {
                if (handStruct.StructType != HandStructType.Head)
                    continue;
                if ((int)handStruct.HandType == (int)gameData.Dir_game || (int)handStruct.HandType == (int)gameData.Dir_player)
                    return false;
                if (Is_ColorHand(handStruct))
                    return false;
            }

            var lastStruct = handData.StructList[handData.LastHand.LastHandFrom];
            if ((int)lastStruct.HandType % 10 == 1 && (int)handData.LastHand.HandType % 10 == 3)
                return false;
            else if ((int)lastStruct.HandType % 10 == 7 && (int)handData.LastHand.HandType % 10 == 7)
                return false;
            else if ((int)handData.LastHand.HandType % 10 == ((int)lastStruct.HandType % 10) + 1)
                return false;
            else
                return true;
        }

        private bool Is_Ipeko()
        {
            if (!menzen)
                return false;
            
            
            for(int i=0; i<4; i++)
            {
                var firstStruct = handData.StructList[i];
                if (firstStruct.StructType == HandStructType.Shunzi)
                {
                    for(int j=i+1;j<5;j++)
                    {
                        var secondStruct = handData.StructList[j];
                        if (secondStruct.StructType == HandStructType.Shunzi && firstStruct.HandType == secondStruct.HandType)
                            return true;
                    }
                }
            }
            return false;
        }

        private bool Is_Ryanpeko()
        {
            if (!menzen)
                return false;


            int count = 0;
            for (int i = 0; i < 4; i++)
            {
                var firstStruct = handData.StructList[i];
                if (firstStruct.StructType == HandStructType.Shunzi)
                {
                    for (int j = i + 1; j < 5; j++)
                    {
                        var secondStruct = handData.StructList[j];
                        if (secondStruct.StructType == HandStructType.Shunzi && firstStruct.HandType == secondStruct.HandType)
                            count += 1;
                    }
                }
            }
            if (count == 2)
                return true;
            else
                return false;
        }

        private bool Is_Tanyao()
        {
            for(int i=0; i<5; i++)
            {
                var handStruct = handData.StructList[i];
                if (Is_RequireHand(handStruct))
                    return false;
                else if (Is_Shunzi(handStruct) && Is_NumberHand(handStruct, 7))
                    return false;
            }
            return true;
        }

        private bool Is_Youngsang()
        {
            if (handData.LastHand.LastHandType == LastHandType.Youngsang)
                return true;
            else
                return false;
        }

        private int Count_Color()
        {
            int count = 0;
            foreach(var handStruct in handData.StructList)
            {
                if (Is_ColorHand(handStruct) && handStruct.StructType != HandStructType.Head)
                    count += 1;
            }
            return count;
        }

        private bool Same_GameDirection()
        {
            foreach(var handStruct in handData.StructList)
            {
                if ((int)handStruct.HandType == (int)gameData.Dir_game && handStruct.StructType != HandStructType.Head)
                    return true;
            }
            return false;
        }

        private bool Same_PlayerDirection()
        {
            foreach (var handStruct in handData.StructList)
            {
                if ((int)handStruct.HandType == (int)gameData.Dir_player && handStruct.StructType != HandStructType.Head)
                    return true;
            }
            return false;
        }

        private bool Is_ShunziWithThreeTypes()
        {
            for(int i=1; i<10; i++)
            {
                int[] count_series = { 0, 0, 0 };
                foreach(var handStruct in handData.StructList)
                {
                    if (Is_Shunzi(handStruct) && Is_NumberHand(handStruct, i))
                        count_series[(int)handStruct.HandType / 10 - 1] += 1;
                }
                if (count_series[0] >= 1 && count_series[1] >= 1 && count_series[2] >= 1)
                    return true;
            }
            return false;
        }

        private bool Is_KeziWithThreeTypes()
        {
            for (int i = 1; i < 10; i++)
            {
                int[] count_series = { 0, 0, 0 };
                foreach (var handStruct in handData.StructList)
                {
                    if (Is_Kezi(handStruct) && Is_NumberHand(handStruct, i))
                        count_series[(int)handStruct.HandType / 10 - 1] += 1;
                }
                if (count_series[0] >= 1 && count_series[1] >= 1 && count_series[2] >= 1)
                    return true;
            }
            return false;
        }

        private bool Is_Chanta()
        {
            int count_require = 0;
            
            foreach (var handStruct in handData.StructList)
            {
                if (Is_RequireHand(handStruct))
                    count_require += 1;
                else if (Is_Shunzi(handStruct) && Is_NumberHand(handStruct,7))
                    count_require += 1;
            }
            if (count_require == 5)
                return true;
            else
                return false;
        }

        private bool Is_IncludingOneToNine()
        {
            int[,] oneToNine = new int[3, 3];
            foreach(var handStruct in handData.StructList)
            {
                if (Is_Shunzi(handStruct) && ((int)handStruct.HandType % 10) % 3 == 1)
                    oneToNine[(int)handStruct.HandType / 10 - 1, ((int)handStruct.HandType % 10) / 3] += 1;
            }
            for(int i=0;i<3;i++)
            {
                if (oneToNine[i, 0] >= 1 && oneToNine[i, 1] >= 1 && oneToNine[i, 2] >= 1)
                    return true;
            }
            return false;
        }

        private int Count_Kezi()
        {
            int count_kezi = 0;
            foreach (var handStruct in handData.StructList)
            {
                if (Is_Kezi(handStruct))
                    count_kezi += 1;
            }
            return count_kezi;
        }

        private bool Is_Honnodu()
        {
            foreach(var handStruct in handData.StructList)
            {
                if (Is_Shunzi(handStruct) || !Is_RequireHand(handStruct))
                    return false;
            }
            return true;
        }

        private bool Is_SmallTripleColor()
        {
            if (Count_Color() != 2)
                return false;

            foreach(var handStruct in handData.StructList)
            {
                if (handStruct.StructType == HandStructType.Head && Is_ColorHand(handStruct))
                    return true;
            }
            return false;
        }

        private int Count_Gangzi()
        {
            int count_gangzi = 0;
            foreach(var handStruct in handData.StructList)
            {
                if (Is_Gangzi(handStruct))
                    count_gangzi += 1;
            }
            return count_gangzi;
        }
        
        private bool Is_IncludingOneOrNine()
        {
            int count_OneOrNine = 0;
            foreach(var handStruct in handData.StructList)
            {
                if (Is_NumberHand(handStruct, 1) || Is_NumberHand(handStruct, 9))
                    count_OneOrNine += 1;
                else if (Is_NumberHand(handStruct, 7) && Is_Shunzi(handStruct))
                    count_OneOrNine += 1;
            }
            if (count_OneOrNine == 5)
                return true;
            else
                return false;
        }

        private bool Is_ComplexOneType()
        {
            for(int i=0; i<4;i++)
            {
                var firstStruct = handData.StructList[i];
                if ((int)firstStruct.HandType < 10)
                    continue;
                for(int j=i+1;j<5;j++)
                {
                    var secondStruct = handData.StructList[j];
                    if ((int)secondStruct.HandType < 10)
                        continue;
                    if ((int)firstStruct.HandType / 10 != (int)secondStruct.HandType / 10)
                        return false;
                }
            }
            return true;
        }

        private int Count_Char()
        {
            int count = 0;
            foreach(var handStruct in handData.StructList)
            {
                if (Is_CharHand(handStruct))
                    count += 1;
            }
            return count;
        }

        private bool Is_PureOneType()
        {
            for (int i = 0; i < 4; i++)
            {
                var firstStruct = handData.StructList[i];
                if ((int)firstStruct.HandType < 10)
                    return false;
                for (int j = i + 1; j < 5; j++)
                {
                    var secondStruct = handData.StructList[j];
                    if ((int)secondStruct.HandType < 10)
                        return false;
                    if ((int)firstStruct.HandType / 10 != (int)secondStruct.HandType / 10)
                        return false;
                }
            }
            return true;
        }

        private bool Is_Smallsahi()
        {
            int count = 0;
            foreach(var handStruct in handData.StructList)
            {
                if (Is_Kezi(handStruct) && (int)handStruct.HandType < 5)
                    count += 1;
                if (handStruct.StructType == HandStructType.Head && (int)handStruct.HandType >= 5)
                    return false;
            }
            if (count == 3)
                return true;
            else
                return false;
        }

        private bool Is_Bigsahi()
        {
            int count = 0;
            foreach (var handStruct in handData.StructList)
            {
                if (Is_Kezi(handStruct) && (int)handStruct.HandType < 5)
                    count += 1;
            }
            if (count == 4)
                return true;
            else
                return false;
        }

        private bool Is_Chungnodu()
        {
            int count = 0;
            foreach(var handStruct in handData.StructList)
            {
                if (Is_Kezi(handStruct) && (Is_NumberHand(handStruct, 1) || Is_NumberHand(handStruct, 9)))
                    count += 1;
                if (handStruct.StructType == HandStructType.Head && ((int)handStruct.HandType < 10 || ((int)handStruct.HandType % 10 > 1 && (int)handStruct.HandType < 9)))
                    return false;
            }
            if (count == 4)
                return true;
            else
                return false;
        }

        private bool Is_OnlyGreen()
        {
            int[] skip = { 6, 8, 9, 10, 32, 33, 34, 36, 38 };
            for(int i=1; i<40; i++)
            {
                if (skip.Contains(i))
                    continue;
                foreach (var handStruct in handData.StructList)
                {
                    if (Count_HandAmount(handStruct, (HandType)i) > 0)
                        return false;
                }
            }
            return true;
            
        }

        private bool Is_ConnectedNine()
        {
            foreach (var handStruct in handData.StructList)
            {
                switch (handStruct.StructType)
                {
                    case HandStructType.Pun:
                        return false;
                    case HandStructType.Chi:
                        return false;
                    case HandStructType.Gangzi_Out:
                        return false;
                }
            }

            int[,] count = new int[3, 9];
            foreach(var handStruct in handData.StructList)
            {
                for(int i=1; i<4; i++)
                {
                    for(int j=1; j<10;j++)
                    {
                        count[i - 1, j - 1] += Count_HandAmount(handStruct, (HandType)(i * 10 + j));
                    }
                }
            }

            for(int i=0; i<3; i++)
            {
                int count_connectedNine = 0;
                for(int j=0; j<9;j++)
                {
                    if (j == 0 || j == 8)
                    {
                        if (count[i, j] >= 3)
                            count_connectedNine += 1;
                    }
                    else if (count[i, j] >= 1)
                        count_connectedNine += 1;
                }
                if (count_connectedNine == 9)
                    return true;
            }
            return false;
        }

        private bool Is_PureConnectedNine()
        {
            int[,] count = new int[3, 9];
            foreach (var handStruct in handData.StructList)
            {
                for (int i = 1; i < 4; i++)
                {
                    for (int j = 1; j < 10; j++)
                    {
                        count[i - 1, j - 1] += Count_HandAmount(handStruct, (HandType)(i * 10 + j));
                    }
                }
            }

            for (int i = 0; i < 3; i++)
            {
                for (int j = 0; j < 9; j++)
                {
                    if (j == 0 || j == 8)
                        count[i, j] -= 3;
                    else if (count[i, j] >= 1)
                        count[i, j] -= 1;
                }
                for(int j=0; j<9; j++)
                {
                    if (count[i, j] < 0)
                        break;
                    else if (count[i, j] == 1 && (HandType)((i + 1) * 10 + j + 1) == handData.LastHand.HandType)
                        return true;
                }
            }
            return false;
        }
    }
}

