using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using TMPro;
using UnityEngine.UI;

namespace Mahjong
{
    public class GameDataIndicator: MonoBehaviour
    {
        [SerializeField] private TMP_Text text;
        [SerializeField] private Transform dora, backDora;
        [SerializeField] private HandObject handObjectPrefab;

        public void UpdateData(GameData gameData)
        {
            text.text = $"{GetDirection(gameData.Dir_game)}, {GetDirection(gameData.Dir_player)}, {Is_Start(gameData.StartPlayer)}" + Is_Rich(gameData);
            DestroyDoraObjects();
            for(int i=0; i<5; i++)
            {
                if(i<gameData.DoraAmount)
                {
                    InstHandObject(GetDora(gameData.DoraList[i]), false);
                    if (gameData.Rich)
                        InstHandObject(GetDora(gameData.BackdoraList[i]), true);
                    else
                        InstHandObject(HandType.Back, true);
                }
                else
                {
                    InstHandObject(HandType.Back, false);
                    InstHandObject(HandType.Back, true);
                }
            }
        }

        public void EndGame()
        {
            text.text = $" ";
            DestroyDoraObjects();
        }

        private void DestroyDoraObjects()
        {
            for (int i = 0; i < dora.childCount; i++)
            {
                Destroy(dora.GetChild(i).gameObject);
                Destroy(backDora.GetChild(i).gameObject);
            }
        }

        private HandType GetDora(HandType type)
        {
            int value = (int)type;
            if (value > 10 && value % 10 == 1)
                return (HandType)(value + 8);
            else if (value == 1)
                return HandType.North;
            else if (value == 5)
                return HandType.Red;
            else
                return (HandType)(value - 1);
        }

        private void InstHandObject(HandType type, bool back)
        {
            var handObject = Instantiate(handObjectPrefab, dora);
            if (back)
                handObject.transform.SetParent(backDora);
            handObject.UpdateType(type);
        }

        private string GetDirection(Direction dir)
        {
            switch(dir)
            {
                case Direction.East:
                    return "";
                case Direction.South:
                    return "";
                case Direction.West:
                    return "";
                case Direction.North:
                    return "";
                default:
                    return "Error";
            }
        }

        private string Is_Start(bool start)
        {
            if (start)
                return "ģ";
            else
                return "";
        }

        private string Is_Rich(GameData gameData)
        {
            if (!gameData.Rich)
                return "";
            else if (!gameData.OneBal)
                return ", ġ";
            else
                return ", ġ Ϲ";
        }
    }

}
