using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using UnityEngine;

namespace Mahjong
{
    public class HandsPanel : MonoBehaviour
    {
        [SerializeField] private HandObject handObjectPrefab;
        [SerializeField] private LastHandObject lastHandPrefab;
        [SerializeField] private GameObject blankObjectPrefab;
        [SerializeField] private Transform ExtraHands;
        public void UpdateHands(HandData handData)
        {
            RemoveRemains();

            for(int i=0; i<handData.StructList.Count; i++)
            {
                var handStruct = handData.StructList[i];
                if(i != handData.LastHand.LastHandFrom)
                {
                    switch (handStruct.StructType)
                    {
                        case HandStructType.Shunzi:
                            InstHandObject(handStruct.HandType, false);
                            InstHandObject(handStruct.HandType + 1, false);
                            InstHandObject(handStruct.HandType + 2, false);
                            break;
                        case HandStructType.Kezi:
                            InstHandObject(handStruct.HandType, false);
                            InstHandObject(handStruct.HandType, false);
                            InstHandObject(handStruct.HandType, false);
                            break;
                        case HandStructType.Chi:
                            InstHandObject(handStruct.HandType, true, true);
                            InstHandObject(handStruct.HandType + 1, false ,true);
                            InstHandObject(handStruct.HandType + 2, false, true);
                            break;
                        case HandStructType.Pun:
                            InstHandObject(handStruct.HandType, true, true);
                            InstHandObject(handStruct.HandType, false, true);
                            InstHandObject(handStruct.HandType, false, true);
                            break;
                        case HandStructType.Gangzi_In:
                            InstHandObject(HandType.Back, false, true);
                            InstHandObject(handStruct.HandType, false, true);
                            InstHandObject(handStruct.HandType, false, true);
                            InstHandObject(HandType.Back, false, true);
                            break;
                        case HandStructType.Gangzi_Out:
                            InstHandObject(handStruct.HandType, true, true);
                            InstHandObject(handStruct.HandType, false, true);
                            InstHandObject(handStruct.HandType, false, true);
                            InstHandObject(handStruct.HandType, false, true);
                            break;
                        case HandStructType.Head:
                            InstHandObject(handStruct.HandType, false);
                            InstHandObject(handStruct.HandType, false);
                            break;
                    }
                }
                else
                {
                    switch (handStruct.StructType)
                    {
                        case HandStructType.Shunzi:
                            for(int j=0; j<3; j++)
                            {
                                var type = handStruct.HandType + j;
                                if (type == handData.LastHand.HandType)
                                    continue;
                                InstHandObject(type, false);
                            }
                            break;
                        case HandStructType.Kezi:
                            InstHandObject(handStruct.HandType, false);
                            InstHandObject(handStruct.HandType, false);
                            break;
                        case HandStructType.Head:
                            InstHandObject(handStruct.HandType, false);
                            break;
                    }
                }
                
            }
            Instantiate(blankObjectPrefab, this.transform);
            InstLastHand(handData.LastHand.HandType);
            SortHands();

        }

        public void RemoveRemains()
        {
            for (int i = 0; i < this.transform.childCount; i++)
            {
                Destroy(this.transform.GetChild(i).gameObject);
            }

            for (int i = 0; i < ExtraHands.childCount; i++)
            {
                Destroy(ExtraHands.GetChild(i).gameObject);
            }
        }

        private void InstHandObject(HandType type, bool rotate, bool extra = false)
        {
            var handObject = Instantiate(handObjectPrefab, this.transform);
            if (extra)
                handObject.transform.SetParent(ExtraHands);
            handObject.UpdateType(type);
            if (rotate)
                handObject.RotateIntoLeft();
        }

        private void SortHands()
        {
            var handObjects = this.transform.GetComponentsInChildren<HandObject>();
            handObjects = handObjects.OrderBy(x => (int)x.type).ToArray();
            for (int i=0; i<handObjects.Length;i++)
            {
                handObjects[i].transform.SetSiblingIndex(i);
            }
        }

        private void InstLastHand(HandType type)
        {
            var handObject = Instantiate(lastHandPrefab, this.transform);
            handObject.UpdateData(GameManager.Instance.Table.GetSpriteof(type), GameManager.Instance.HandData.LastHand.LastHandType);
        }
    }
}
